@@ -5,10 +5,13 @@ var dns = require('dns');
 
 var IOWatcher    = process.binding('io_watcher').IOWatcher;
 var binding      = process.binding('net');
+var constants    = process.binding('constants');
+
 var socket       = binding.socket;
 var recvfrom     = binding.recvfrom;
 var close        = binding.close;
-var ENOENT       = binding.ENOENT;
+
+var ENOENT       = constants.ENOENT;
 
 function isPort (x) { return parseInt(x) >= 0; }
 var pool = null;
