@@ -388,17 +388,18 @@ fs.chownSync = function(path, uid, gid) {
   return binding.chown(path, uid, gid);
 };
 
-function writeAll (fd, buffer, callback) {
-  fs.write(fd, buffer, 0, buffer.length, null, function (writeErr, written) {
+function writeAll (fd, buffer, offset, length, callback) {
+  // write(fd, buffer, offset, length, position, callback)
+  fs.write(fd, buffer, offset, length, offset, function (writeErr, written) {
     if (writeErr) {
       fs.close(fd, function () {
         if (callback) callback(writeErr);
       });
     } else {
-      if (written === buffer.length) {
+      if (written === length) {
         fs.close(fd, callback);
       } else {
-        writeAll(fd, buffer.slice(written), callback);
+        writeAll(fd, buffer, offset+written, length-written, callback);
       }
     }
   });
@@ -413,18 +414,21 @@ fs.writeFile = function (path, data, encoding_, callback) {
       if (callback) callback(openErr);
     } else {
       var buffer = Buffer.isBuffer(data) ? data : new Buffer(data, encoding);
-      writeAll(fd, buffer, callback);
+      writeAll(fd, buffer, 0, buffer.length, callback);
     }
   });
 };
 
 fs.writeFileSync = function (path, data, encoding) {
-  encoding = encoding || "utf8"; // default to utf8
   var fd = fs.openSync(path, "w");
+  if (!Buffer.isBuffer(data)) {
+    data = new Buffer(data, encoding || "utf8")
+  }
   var written = 0;
-  while (written < data.length) {
-    written += fs.writeSync(fd, data, 0, encoding);
-    data = data.slice(written);
+  var length = data.length;
+  //writeSync(fd, buffer, offset, length, position)
+  while (written < length) {
+    written += fs.writeSync(fd, data, written, length-written, written);
   }
   fs.closeSync(fd);
 };
@@ -484,8 +488,8 @@ function realpathSync (p) {
   if (p.charAt(0) !== '/') {
     p = path.join(process.cwd(), p);
   }
-  p = p.split('/');
-  var buf = [ '' ];
+  p = path.split(p);
+  var buf = [];
   var seenLinks = {};
   var knownHard = {};
   // walk down the path, swapping out linked pathparts for their real
@@ -495,7 +499,7 @@ function realpathSync (p) {
   for (var i = 0; i < p.length; i ++) {
     // skip over empty path parts.
     if (p[i] === '') continue;
-    var part = buf.join('/')+'/'+p[i];
+    var part = path.join.apply(path, buf.concat(p[i]));
     if (knownHard[part]) {
       buf.push( p[i] );
       continue;
@@ -508,24 +512,29 @@ function realpathSync (p) {
       continue;
     }
     var id = stat.dev.toString(32)+':'+stat.ino.toString(32);
-    if (seenLinks[id]) throw new Error("cyclic link at "+part);
-    seenLinks[id] = true;
-    var target = fs.readlinkSync(part);
+    if (!seenLinks[id]) {
+      fs.statSync(part);
+      seenLinks[id] = fs.readlinkSync(part);
+    }
+    var target = seenLinks[id];
     if (target.charAt(0) === '/') {
       // absolute. Start over.
-      buf = [''];
-      p = path.normalizeArray(target.split('/').concat(p.slice(i + 1)));
-      i = 0;
+      buf = [];
+      p = path.normalizeArray(path.split(target).concat(p.slice(i + 1)));
+      i = -1;
       continue;
     }
     // not absolute.  join and splice.
-    target = target.split('/');
+    if (i === 0 && p[i].charAt(0) === "/") {
+      target = "/"+target;
+    }
+    target = path.split(target);
     Array.prototype.splice.apply(p, [i, 1].concat(target));
     p = path.normalizeArray(p);
-    i = 0;
-    buf = [''];
+    i = -1;
+    buf = [];
   }
-  return buf.join('/') || '/';
+  return path.join(buf.join('/') || '/');
 }
 
 
@@ -533,15 +542,15 @@ function realpath (p, cb) {
   if (p.charAt(0) !== '/') {
     p = path.join(process.cwd(), p);
   }
-  p = p.split('/');
-  var buf = [ '' ];
+  p = path.split(p);
+  var buf = [];
   var seenLinks = {};
   var knownHard = {};
   // walk down the path, swapping out linked pathparts for their real
   // values, and pushing non-link path bits onto the buffer.
   // then return the buffer.
   // NB: path.length changes.
-  var i = 0;
+  var i = -1;
   var part;
   LOOP();
   function LOOP () {
@@ -549,7 +558,7 @@ function realpath (p, cb) {
     if (!(i < p.length)) return exit();
     // skip over empty path parts.
     if (p[i] === '') return process.nextTick(LOOP);
-    part = buf.join('/')+'/'+p[i];
+    part = path.join(buf.join('/')+'/'+p[i]);
     if (knownHard[part]) {
       buf.push( p[i] );
       return process.nextTick(LOOP);
@@ -565,29 +574,36 @@ function realpath (p, cb) {
       return process.nextTick(LOOP);
     }
     var id = stat.dev.toString(32)+':'+stat.ino.toString(32);
-    if (seenLinks[id]) return cb(new Error("cyclic link at "+part));
-    seenLinks[id] = true;
-    fs.readlink(part, gotTarget);
+    if (seenLinks[id]) return gotTarget(null, seenLinks[id]);
+    fs.stat(part, function (er) {
+      if (er) return cb(er)
+      fs.readlink(part, function (er, target) {
+        gotTarget(er, seenLinks[id] = target);
+      });
+    })
   }
   function gotTarget (er, target) {
     if (er) return cb(er);
     if (target.charAt(0) === '/') {
       // absolute. Start over.
-      buf = [''];
-      p = path.normalizeArray(target.split('/').concat(p.slice(i + 1)));
-      i = 0;
+      buf = [];
+      p = path.normalizeArray(path.split(target).concat(p.slice(i + 1)));
+      i = -1;
       return process.nextTick(LOOP);
     }
     // not absolute.  join and splice.
-    target = target.split('/');
+    if (i === 0 && p[i].charAt(0) === "/") {
+      target = "/"+target;
+    }
+    target = path.split(target);
     Array.prototype.splice.apply(p, [i, 1].concat(target));
     p = path.normalizeArray(p);
-    i = 0;
-    buf = [''];
+    i = -1;
+    buf = [];
     return process.nextTick(LOOP);
   }
   function exit () {
-    cb(null, buf.join('/') || '/');
+    cb(null, path.join(buf.join('/') || '/'));
   }
 }
 
@@ -637,7 +653,7 @@ var ReadStream = fs.ReadStream = function(path, options) {
     } else if (this.start > this.end) {
       this.emit('error', new Error('start must be <= end'));
     } else {
-      this.firstRead = true;
+      this._firstRead = true;
     }
   }
 
@@ -678,8 +694,9 @@ ReadStream.prototype._read = function () {
     allocNewPool();
   }
 
-  if (self.start !== undefined && self.firstRead) {
+  if (self.start !== undefined && self._firstRead) {
     self.pos = self.start;
+    self._firstRead = false;
   }
 
   // Grab another reference to the pool in the case that while we're in the
@@ -725,10 +742,7 @@ ReadStream.prototype._read = function () {
     self._read();
   }
 
-  // pass null for position after we've seeked to the start of a range read
-  // always pass null on a non-range read
-  fs.read(self.fd, pool, pool.used, toRead, (self.firstRead ? self.pos : null), afterRead);
-  self.firstRead = false;
+  fs.read(self.fd, pool, pool.used, toRead, self.pos, afterRead);
 
   if (self.pos !== undefined) {
     self.pos += toRead;
