@@ -743,9 +743,9 @@ function httpSocketSetup (socket) {
 
 function Server (requestListener) {
   if (!(this instanceof Server)) return new Server(requestListener);
-  net.Server.call(this);
+  net.Server.call(this, { allowHalfOpen: true });
 
-  if(requestListener){
+  if (requestListener) {
     this.addListener("request", requestListener);
   }
 
@@ -877,7 +877,7 @@ function connectionListener (socket) {
 
 function Client ( ) {
   if (!(this instanceof Client)) return new Client();
-  net.Stream.call(this);
+  net.Stream.call(this, { allowHalfOpen: true });
   var self = this;
 
   httpSocketSetup(self);
