@@ -1,8 +1,6 @@
 var util = require("util");
-var fs = require("fs");
 var events = require("events");
 var stream = require("stream");
-var dns = require('dns');
 
 var kMinPoolSpace = 128;
 var kPoolSize = 40*1024;
@@ -24,7 +22,7 @@ var binding = process.binding('net');
 var FreeList = require('freelist').FreeList;
 
 var IOWatcher   = process.binding('io_watcher').IOWatcher;
-var Timer       = process.binding('timer').Timer;
+var constants   = process.binding('constants');
 var assert      = process.assert;
 
 var socket      = binding.socket;
@@ -44,169 +42,14 @@ var getsockname = binding.getsockname;
 var errnoException = binding.errnoException;
 var sendMsg     = binding.sendMsg;
 var recvMsg     = binding.recvMsg;
-var EINPROGRESS = binding.EINPROGRESS;
-var ENOENT      = binding.ENOENT;
-var EMFILE      = binding.EMFILE;
+
+var EINPROGRESS = constants.EINPROGRESS;
+var ENOENT      = constants.ENOENT;
+var EMFILE      = constants.EMFILE;
+
 var END_OF_FILE = 42;
 var SecureContext, SecureStream; // lazy loaded
 
-// IDLE TIMEOUTS
-//
-// Because often many sockets will have the same idle timeout we will not
-// use one timeout watcher per socket. It is too much overhead.  Instead
-// we'll use a single watcher for all sockets with the same timeout value
-// and a linked list. This technique is described in the libev manual:
-// http://pod.tst.eu/http://cvs.schmorp.de/libev/ev.pod#Be_smart_about_timeouts
-
-
-var timeout = new (function () {
-  // Object containing all lists, timers
-  // key = time in milliseconds
-  // value = list
-  var lists = {};
-
-  // show the most idle socket
-  function peek (list) {
-    if (list._idlePrev == list) return null;
-    return list._idlePrev;
-  }
-
-
-  // remove the most idle socket from the list
-  function shift (list) {
-    var first = list._idlePrev;
-    remove(first);
-    return first;
-  }
-
-
-  // remove a socket from its list
-  function remove (socket) {
-    socket._idleNext._idlePrev = socket._idlePrev;
-    socket._idlePrev._idleNext = socket._idleNext;
-  }
-
-
-  // remove a socket from its list and place at the end.
-  function append (list, socket) {
-    remove(socket);
-    socket._idleNext = list._idleNext;
-    socket._idleNext._idlePrev = socket;
-    socket._idlePrev = list;
-    list._idleNext = socket;
-  }
-
-
-  function normalize (msecs) {
-    if (!msecs || msecs <= 0) return 0;
-    // round up to one sec
-    if (msecs < 1000) return 1000;
-    // round down to nearest second.
-    return msecs - (msecs % 1000);
-  }
-
-  // the main function - creates lists on demand and the watchers associated
-  // with them.
-  function insert (socket, msecs) {
-    socket._idleStart = new Date();
-    socket._idleTimeout = msecs;
-
-    if (!msecs) return;
-
-    var list;
-
-    if (lists[msecs]) {
-      list = lists[msecs];
-    } else {
-      list = new Timer();
-      list._idleNext = list;
-      list._idlePrev = list;
-
-      lists[msecs] = list;
-
-      list.callback = function () {
-        debug('timeout callback ' + msecs);
-        // TODO - don't stop and start the watcher all the time.
-        // just set its repeat
-        var now = new Date();
-        debug("now: " + now);
-        var first;
-        while (first = peek(list)) {
-          var diff = now - first._idleStart;
-          if (diff < msecs) {
-            list.again(msecs - diff);
-            debug(msecs + ' list wait because diff is '  + diff);
-            return;
-          } else {
-            remove(first);
-            assert(first != peek(list));
-            first.emit('timeout');
-          }
-        }
-        debug(msecs + ' list empty');
-        assert(list._idleNext == list); // list is empty
-        list.stop();
-      };
-    }
-
-    if (list._idleNext == list) {
-      // if empty (re)start the timer
-      list.again(msecs);
-    }
-
-    append(list, socket);
-    assert(list._idleNext != list); // list is not empty
-  }
-
-
-  var unenroll = this.unenroll = function (socket) {
-    if (socket._idleNext) {
-      socket._idleNext._idlePrev = socket._idlePrev;
-      socket._idlePrev._idleNext = socket._idleNext;
-
-      var list = lists[socket._idleTimeout];
-      // if empty then stop the watcher
-      //debug('unenroll');
-      if (list && list._idlePrev == list) {
-        //debug('unenroll: list empty');
-        list.stop();
-      }
-    }
-  };
-
-
-  // Does not start the time, just sets up the members needed.
-  this.enroll = function (socket, msecs) {
-    // if this socket was already in a list somewhere
-    // then we should unenroll it from that
-    if (socket._idleNext) unenroll(socket);
-
-    socket._idleTimeout = msecs;
-    socket._idleNext = socket;
-    socket._idlePrev = socket;
-  };
-
-  // call this whenever the socket is active (not idle)
-  // it will reset its timeout.
-  this.active = function (socket) {
-    var msecs = socket._idleTimeout;
-    if (msecs) {
-      var list = lists[msecs];
-      if (socket._idleNext == socket) {
-        insert(socket, msecs);
-      } else {
-        // inline append
-        socket._idleStart = new Date();
-        socket._idleNext._idlePrev = socket._idlePrev;
-        socket._idlePrev._idleNext = socket._idleNext;
-        socket._idleNext = list._idleNext;
-        socket._idleNext._idlePrev = socket;
-        socket._idlePrev = list;
-        list._idleNext = socket;
-      }
-    }
-  };
-})();
 
 var ioWatchers = new FreeList("iowatcher", 100, function () {
   return new IOWatcher();
@@ -247,16 +90,6 @@ function allocEmptyBuffer () {
     emptyBuffer.length = 0;
 }
 
-function _doFlush () {
-  var socket = this.socket;
-  // Stream becomes writable on connect() but don't flush if there's
-  // nothing actually to write
-  if (socket.flush()) {
-    if (socket._events && socket._events['drain']) socket.emit("drain");
-    if (socket.ondrain) socket.ondrain(); // Optimization
-  }
-}
-
 function setImplmentationMethods (self) {
   function noData(buf, off, len) {
     return !buf ||
@@ -333,7 +166,7 @@ function setImplmentationMethods (self) {
 
       if (secureLen == -1) {
         // Check our read again for secure handshake
-        self._readWatcher.callback();
+        self._onReadable();
       } else {
         oldWrite(securePool, 0, secureLen, fd, flags);
       }
@@ -380,7 +213,7 @@ function setImplmentationMethods (self) {
 
       if (self.secureStream.clearPending()) {
         process.nextTick(function () {
-          if (self._readWatcher) self._readWatcher.callback();
+          if (self.readable) self._onReadable();
         });
       }
 
@@ -436,73 +269,28 @@ function setImplmentationMethods (self) {
   }
 };
 
+
+function onReadable (readable, writeable) {
+  assert(this.socket);
+  var socket = this.socket;
+  socket._onReadable(true);
+}
+
+
+function onWritable (readable, writeable) {
+  assert(this.socket);
+  var socket = this.socket;
+  if (socket._connecting) {
+    socket._onConnect();
+  } else {
+    socket._onWritable();
+  }
+}
+
 function initStream (self) {
   self._readWatcher = ioWatchers.alloc();
-  self._readWatcher.callback = function () {
-    // If this is the first recv (pool doesn't exist) or we've used up
-    // most of the pool, allocate a new one.
-    if (!pool || pool.length - pool.used < kMinPoolSpace) {
-      // discard the old pool. Can't add to the free list because
-      // users might have refernces to slices on it.
-      pool = null;
-      allocNewPool();
-    }
-
-    //debug('pool.used ' + pool.used);
-    var bytesRead;
-
-    try {
-      bytesRead = self._readImpl(pool,
-                                 pool.used,
-                                 pool.length - pool.used,
-                                 (arguments.length > 0));
-    } catch (e) {
-      self.destroy(e);
-      return;
-    }
-
-    // Note that some _readImpl() implementations return -1 bytes
-    // read as an indication not to do any processing on the result
-    // (but not an error).
-
-    if (bytesRead === 0) {
-      self.readable = false;
-      self._readWatcher.stop();
-
-      if (!self.writable) self.destroy();
-      // Note: 'close' not emitted until nextTick.
-
-      if (self._events && self._events['end']) self.emit('end');
-      if (self.onend) self.onend();
-    } else if (bytesRead > 0) {
-
-      timeout.active(self);
-
-      var start = pool.used;
-      var end = pool.used + bytesRead;
-      pool.used += bytesRead;
-
-      if (self._decoder) {
-        // emit String
-        var string = self._decoder.write(pool.slice(start, end));
-        if (string.length) self.emit('data', string);
-      } else {
-        // emit buffer
-        if (self._events && self._events['data']) {
-          // emit a slice
-          self.emit('data', pool.slice(start, end));
-        }
-      }
-
-      // Optimization: emit the original buffer with end points
-      if (self.ondata) self.ondata(pool, start, end);
-    } else if (bytesRead == -2) {
-      // Temporary fix - need SSL refactor.
-      // -2 originates from SecureStream::ReadExtract
-      self.destroy(new Error('openssl read error'));
-      return false;
-    }
-  };
+  self._readWatcher.socket = self;
+  self._readWatcher.callback = onReadable;
   self.readable = false;
 
   // Queue of buffers and string that need to be written to socket.
@@ -512,20 +300,33 @@ function initStream (self) {
 
   self._writeWatcher = ioWatchers.alloc();
   self._writeWatcher.socket = self;
-  self._writeWatcher.callback = _doFlush;
+  self._writeWatcher.callback = onWritable;
   self.writable = false;
 }
 
-function Stream (fd, type) {
-  if (!(this instanceof Stream)) return new Stream(fd, type);
+// Deprecated API: Stream(fd, type)
+// New API: Stream({ fd: 10, type: "unix", allowHalfOpen: true })
+function Stream (options) {
+  if (!(this instanceof Stream)) return new Stream(arguments[0], arguments[1]);
   stream.Stream.call(this);
 
   this.fd = null;
   this.type = null;
   this.secure = false;
+  this.allowHalfOpen = false;
 
-  if (parseInt(fd, 10) >= 0) {
-    this.open(fd, type);
+  if (typeof options == "object") {
+    this.fd = options.fd !== undefined ? parseInt(options.fd, 10) : null;
+    this.type = options.type || null;
+    this.secure = options.secure ||  false;
+    this.allowHalfOpen = options.allowHalfOpen || false;
+  } else if (typeof options  == "number") {
+    this.fd = arguments[0];
+    this.type = arguments[1];
+  }
+
+  if (parseInt(this.fd, 10) >= 0) {
+    this.open(this.fd, this.type);
   } else {
     setImplmentationMethods(this);
   }
@@ -534,6 +335,11 @@ util.inherits(Stream, stream.Stream);
 exports.Stream = Stream;
 
 
+Stream.prototype._onTimeout = function () {
+  this.emit('timeout');
+};
+
+
 Stream.prototype.setSecure = function (credentials) {
   // Do we have openssl crypto?
   try {
@@ -757,7 +563,7 @@ Stream.prototype._writeOut = function (data, encoding, fd) {
 
   debug('wrote ' + bytesWritten + ' to socket. [fd, off, len] = ' + JSON.stringify([this.fd, off, len]) + "\n");
 
-  timeout.active(this);
+  require('timers').active(this);
 
   if (bytesWritten == len) {
     // awesome. sent to buffer.
@@ -851,37 +657,118 @@ function doConnect (socket, port, host) {
   // Manual Page connect(2) under the error code EINPROGRESS.
   socket._writeWatcher.set(socket.fd, false, true);
   socket._writeWatcher.start();
-  socket._writeWatcher.callback = function () {
-    var errno = socketError(socket.fd);
-    if (errno == 0) {
-      // connection established
-      socket._connecting = false;
-      socket.resume();
-      socket.readable = socket.writable = true;
-      socket._writeWatcher.callback = _doFlush;
-      try {
-        socket.emit('connect');
-      } catch (e) {
-        socket.destroy(e);
-        return;
-      }
-
-
-      if (socket._writeQueue && socket._writeQueue.length) {
-        // Flush socket in case any writes are queued up while connecting.
-        // ugly
-        _doFlush.call(socket._writeWatcher);
-      }
-
-    } else if (errno != EINPROGRESS) {
-      socket.destroy(errnoException(errno, 'connect'));
-    }
-  };
 }
 
+
 function toPort (x) { return (x = Number(x)) >= 0 ? x : false; }
 
 
+Stream.prototype._onConnect = function () {
+  var errno = socketError(this.fd);
+  if (errno == 0) {
+    // connection established
+    this._connecting = false;
+    this.resume();
+    this.readable = this.writable = true;
+    try {
+      this.emit('connect');
+    } catch (e) {
+      this.destroy(e);
+      return;
+    }
+
+
+    if (this._writeQueue && this._writeQueue.length) {
+      // Flush this in case any writes are queued up while connecting.
+      this._onWritable();
+    }
+
+  } else if (errno != EINPROGRESS) {
+    this.destroy(errnoException(errno, 'connect'));
+  }
+};
+
+
+Stream.prototype._onWritable = function () {
+  // Stream becomes writable on connect() but don't flush if there's
+  // nothing actually to write
+  if (this.flush()) {
+    if (this._events && this._events['drain']) this.emit("drain");
+    if (this.ondrain) this.ondrain(); // Optimization
+  }
+};
+
+
+Stream.prototype._onReadable = function (calledByIOWatcher) {
+  var self = this;
+
+  // If this is the first recv (pool doesn't exist) or we've used up
+  // most of the pool, allocate a new one.
+  if (!pool || pool.length - pool.used < kMinPoolSpace) {
+    // discard the old pool. Can't add to the free list because
+    // users might have refernces to slices on it.
+    pool = null;
+    allocNewPool();
+  }
+
+  //debug('pool.used ' + pool.used);
+  var bytesRead;
+
+  try {
+    bytesRead = self._readImpl(pool,
+                               pool.used,
+                               pool.length - pool.used,
+                               calledByIOWatcher);
+  } catch (e) {
+    self.destroy(e);
+    return;
+  }
+
+  // Note that some _readImpl() implementations return -1 bytes
+  // read as an indication not to do any processing on the result
+  // (but not an error).
+
+  if (bytesRead === 0) {
+    self.readable = false;
+    self._readWatcher.stop();
+
+    if (!self.writable) self.destroy();
+    // Note: 'close' not emitted until nextTick.
+
+    if (!self.allowHalfOpen) self.end();
+    if (self._events && self._events['end']) self.emit('end');
+    if (self.onend) self.onend();
+  } else if (bytesRead > 0) {
+
+    require('timers').active(self);
+
+    var start = pool.used;
+    var end = pool.used + bytesRead;
+    pool.used += bytesRead;
+
+    if (self._decoder) {
+      // emit String
+      var string = self._decoder.write(pool.slice(start, end));
+      if (string.length) self.emit('data', string);
+    } else {
+      // emit buffer
+      if (self._events && self._events['data']) {
+        // emit a slice
+        self.emit('data', pool.slice(start, end));
+      }
+    }
+
+    // Optimization: emit the original buffer with end points
+    if (self.ondata) self.ondata(pool, start, end);
+  } else if (bytesRead == -2) {
+    // Temporary fix - need SSL refactor.
+    // -2 originates from SecureStream::ReadExtract
+    self.destroy(new Error('openssl read error'));
+    return false;
+  }
+};
+
+
 // var stream = new Stream();
 // stream.connect(80)               - TCP connect to port 80 on the localhost
 // stream.connect(80, 'nodejs.org') - TCP connect to port 80 on nodejs.org
@@ -892,7 +779,7 @@ Stream.prototype.connect = function () {
   if (self.fd) throw new Error('Stream already opened');
   if (!self._readWatcher) throw new Error('No readWatcher');
 
-  timeout.active(socket);
+  require('timers').active(socket);
 
   self._connecting = true; // set false in doConnect
   self.writable = true;
@@ -907,7 +794,7 @@ Stream.prototype.connect = function () {
     doConnect(self, arguments[0]);
   } else {
     // TCP
-    dns.lookup(arguments[1], function (err, ip, addressType) {
+    require('dns').lookup(arguments[1], function (err, ip, addressType) {
       if (err) {
         self.emit('error', err);
       } else {
@@ -940,10 +827,10 @@ Stream.prototype.setKeepAlive = function (enable, time) {
 
 Stream.prototype.setTimeout = function (msecs) {
   if (msecs > 0) {
-    timeout.enroll(this, msecs);
-    if (this.fd) { timeout.active(this); }
+    require('timers').enroll(this, msecs);
+    if (this.fd) { require('timers').active(this); }
   } else if (msecs === 0) {
-    timeout.unenroll(this);
+    require('timers').unenroll(this);
   }
 };
 
@@ -972,17 +859,19 @@ Stream.prototype.destroy = function (exception) {
 
   if (this._writeWatcher) {
     this._writeWatcher.stop();
+    this._writeWatcher.socket = null;
     ioWatchers.free(this._writeWatcher);
     this._writeWatcher = null;
   }
 
   if (this._readWatcher) {
     this._readWatcher.stop();
+    this._readWatcher.socket = null;
     ioWatchers.free(this._readWatcher);
     this._readWatcher = null;
   }
 
-  timeout.unenroll(this);
+  require('timers').unenroll(this);
 
   if (this.secure) {
     this.secureStream.close();
@@ -1027,8 +916,8 @@ Stream.prototype._shutdown = function () {
 
 Stream.prototype.end = function (data, encoding) {
   if (this.writable) {
-    if (data) this.write(data, encoding);
     if (this._writeQueueLast() !== END_OF_FILE) {
+      if (data) this.write(data, encoding);
       this._writeQueue.push(END_OF_FILE);
       if (!this._connecting) {
         this.flush();
@@ -1038,60 +927,68 @@ Stream.prototype.end = function (data, encoding) {
 };
 
 
-function Server (listener) {
-  if (!(this instanceof Server)) return new Server(listener);
+function Server (/* [ options, ] listener */) {
+  if (!(this instanceof Server)) return new Server(arguments[0], arguments[1]);
   events.EventEmitter.call(this);
   var self = this;
 
-  if (listener) {
-    self.addListener('connection', listener);
+  var options = {};
+  if (typeof arguments[0] == "object") {
+    options = arguments[0];
+  }
+
+  // listener: find the last argument that is a function
+  for (var l = arguments.length - 1; l >= 0; l--) {
+    if (typeof arguments[l] == "function") {
+      self.addListener('connection', arguments[l]);
+    }
+    if (arguments[l] !== undefined) break;
   }
 
   self.connections = 0;
 
-  self.paused = false;
-  self.pauseTimeout = 1000;
-
-  function pause () {
-    // We've hit the maximum file limit. What to do?
-    // Let's try again in 1 second.
-    self.watcher.stop();
-
-    // If we're already paused, don't do another timeout.
-    if (self.paused) return;
-
-    setTimeout(function () {
-      self.paused = false;
-      // Make sure we haven't closed in the interim
-      if (typeof self.fd  != 'number') return;
-      self.watcher.start();
-    }, self.pauseTimeout);
-  }
+  self.allowHalfOpen = options.allowHalfOpen || false;
 
   self.watcher = new IOWatcher();
   self.watcher.host = self;
   self.watcher.callback = function () {
+    // Just in case we don't have a dummy fd.
+    getDummyFD();
+
+    if (self._pauseTimer) {
+      // Somehow the watcher got started again. Need to wait until
+      // the timer finishes.
+      self.watcher.stop();
+    }
+
     while (self.fd) {
       try {
         var peerInfo = accept(self.fd);
       } catch (e) {
-        if (e.errno == EMFILE) {
-          pause();
-          return;
-        }
-        throw e;
+        if (e.errno != EMFILE) throw e;
+
+        // Gracefully reject pending clients by freeing up a file
+        // descriptor.
+        rescueEMFILE(function() {
+          self._rejectPending();
+        });
+        return;
       }
       if (!peerInfo) return;
 
       if (self.maxConnections && self.connections >= self.maxConnections) {
-        // Accept and close the connection.
+        // Close the connection we just had
         close(peerInfo.fd);
+        // Reject all other pending connectins.
+        self._rejectPending();
         return;
       }
 
       self.connections++;
 
-      var s = new Stream(peerInfo.fd, self.type);
+      var s = new Stream({ fd: peerInfo.fd,
+                           type: self.type,
+                           allowHalfOpen: self.allowHalfOpen });
       s.remoteAddress = peerInfo.address;
       s.remotePort = peerInfo.port;
       s.type = self.type;
@@ -1115,8 +1012,49 @@ util.inherits(Server, events.EventEmitter);
 exports.Server = Server;
 
 
-exports.createServer = function (listener) {
-  return new Server(listener);
+exports.createServer = function () {
+  return new Server(arguments[0], arguments[1]);
+};
+
+
+// Just stop trying to accepting connections for a while.
+// Useful for throttling against DoS attacks.
+Server.prototype.pause = function (msecs) {
+  // We're already paused.
+  if (this._pauseTimer) return;
+
+  var self = this;
+  msecs = msecs || 1000;
+
+  this.watcher.stop();
+
+  // Wait a second before accepting more.
+  this._pauseTimer = setTimeout(function () {
+    // Our fd should still be there. If someone calls server.close() then
+    // the pauseTimer should be cleared.
+    assert(parseInt(self.fd) >= 0);
+    self._pauseTimer = null;
+    self.watcher.start();
+  }, msecs);
+};
+
+
+Server.prototype._rejectPending = function () {
+  var self = this;
+  var acceptCount = 0;
+  // Accept and close the waiting clients one at a time.
+  // Single threaded programming ftw.
+  while (true) {
+    peerInfo = accept(this.fd);
+    if (!peerInfo) return;
+    close(peerInfo.fd);
+
+    // Don't become DoS'd by incoming requests
+    if (++acceptCount > 50) {
+      this.pause();
+      return;
+    }
+  }
 };
 
 
@@ -1138,14 +1076,21 @@ Server.prototype.listen = function () {
   }
 
   var port = toPort(arguments[0]);
-  if (port === false) {
+
+  if (arguments.length == 0 || typeof arguments[0] == 'function') {
+    // Don't bind(). OS will assign a port with INADDR_ANY.
+    // The port can be found with server.address()
+    self.type = 'tcp4';
+    self.fd = socket(self.type);
+    self._doListen(port);
+  } else if (port === false) {
     // the first argument specifies a path
     self.fd = socket('unix');
     self.type = 'unix';
     var path = arguments[0];
     self.path = path;
     // unlink sockfile if it exists
-    fs.stat(path, function (err, r) {
+    require('fs').stat(path, function (err, r) {
       if (err) {
         if (err.errno == ENOENT) {
           self._doListen(path);
@@ -1156,22 +1101,16 @@ Server.prototype.listen = function () {
         if (!r.isSocket()) {
           throw new Error("Non-socket exists at  " + path);
         } else {
-          fs.unlink(path, function (err) {
+          require('fs').unlink(path, function (err) {
             if (err) throw err;
             self._doListen(path);
           });
         }
       }
     });
-  } else if (!arguments[1]) {
-    // Don't bind(). OS will assign a port with INADDR_ANY.
-    // The port can be found with server.address()
-    self.type = 'tcp4';
-    self.fd = socket(self.type);
-    self._doListen(port);
   } else {
     // the first argument is the port, the second an IP
-    dns.lookup(arguments[1], function (err, ip, addressType) {
+    require('dns').lookup(arguments[1], function (err, ip, addressType) {
       if (err) {
         self.emit('error', err);
       } else {
@@ -1202,6 +1141,9 @@ Server.prototype._startWatcher = function () {
 Server.prototype._doListen = function () {
   var self = this;
 
+  // Ensure we have a dummy fd for EMFILE conditions.
+  getDummyFD();
+
   try {
     bind(self.fd, arguments[0], arguments[1]);
   } catch (err) {
@@ -1243,11 +1185,47 @@ Server.prototype.close = function () {
   close(self.fd);
   self.fd = null;
 
+  if (self._pauseTimer) {
+    clearTimeout(self._pauseTimer);
+    self._pauseTimer = null;
+  }
+
   if (self.type === "unix") {
-    fs.unlink(self.path, function () {
+    require('fs').unlink(self.path, function () {
       self.emit("close");
     });
   } else {
     self.emit("close");
   }
 };
+
+
+var dummyFD = null;
+var lastEMFILEWarning = 0;
+// Ensures to have at least on free file-descriptor free.
+// callback should only use 1 file descriptor and close it before end of call
+function rescueEMFILE(callback) {
+  // Output a warning, but only at most every 5 seconds.
+  var now = new Date();
+  if (now - lastEMFILEWarning > 5000) {
+    console.error("(node) Hit max file limit. Increase 'ulimit -n'.");
+    lastEMFILEWarning = now;
+  }
+
+  if (dummyFD) {
+    close(dummyFD);
+    dummyFD = null;
+    callback();
+    getDummyFD();
+  }
+}
+
+function getDummyFD() {
+  if (!dummyFD) {
+    try {
+      dummyFD = socket("tcp");
+    } catch (e) {
+      dummyFD = null;
+    }
+  }
+}
