@@ -3,7 +3,8 @@ var util = require("util");
 var sysWarning;
 if (!sysWarning) {
   sysWarning = "The 'sys' module is now called 'util'. It should have a similar interface.";
-  util.error(sysWarning);
+  // Uncomment in 2011
+  //util.error(sysWarning);
 }
 
 exports.print = util.print;
@@ -15,4 +16,4 @@ exports.p = util.p;
 exports.log = util.log;
 exports.exec = util.exec;
 exports.pump = util.pump;
-exports.inherits = util.inherits;
\ No newline at end of file
+exports.inherits = util.inherits;
