@@ -272,22 +272,31 @@ function urlResolveObject (source, relative) {
       else if (dir !== ".") dirs.push(dir);
     });
 
-    if (mustEndAbs && dirs[0] !== "") {
+    if (mustEndAbs && dirs[0] !== "" && (!dirs[0] || dirs[0].charAt(0) !== "/")) {
       dirs.unshift("");
     }
     srcPath = dirs;
   }
-  if (hasTrailingSlash && (srcPath.length < 2 || srcPath.slice(-1)[0] !== "")) srcPath.push("");
+  if (hasTrailingSlash && (srcPath.join("/").substr(-1) !== "/")) {
+    srcPath.push("");
+  }
+
+  var isAbsolute = srcPath[0] === "" || (srcPath[0] && srcPath[0].charAt(0) === "/");
 
   // put the host back
-  if ( psychotic ) source.host = srcPath[0] === "" ? "" : srcPath.shift();
+  if ( psychotic ) {
+    source.host = isAbsolute ? "" : srcPath.shift();
+  }
 
   mustEndAbs = mustEndAbs || (source.host && srcPath.length);
 
-  if (mustEndAbs && srcPath[0] !== "") srcPath.unshift("");
+  if (mustEndAbs && !isAbsolute) {
+    srcPath.unshift("");
+  }
 
   source.pathname = srcPath.join("/");
 
+
   return source;
 };
 
