@@ -126,21 +126,16 @@ var module = (function () {
   var pathModule = createInternalModule('path', pathFn);
   var path = pathModule.exports;
 
-  // The paths that the user has specifically asked for.  Highest priority.
-  // This is what's hung on require.paths.
-  var modulePaths = [];
-  if (process.env.NODE_PATH) {
-    modulePaths = process.env.NODE_PATH.split(":");
+  var modulePaths = [path.join(process.execPath, "..", "..", "lib", "node")];
+
+  if (process.env["HOME"]) {
+    modulePaths.unshift(path.join(process.env["HOME"], ".node_libraries"));
+    modulePaths.unshift(path.join(process.env["HOME"], ".node_modules"));
   }
 
-  // The default global paths that are always checked.
-  // Lowest priority.
-  var defaultPaths = [];
-  if (process.env.HOME) {
-    defaultPaths.push(path.join(process.env.HOME, ".node_modules"));
-    defaultPaths.push(path.join(process.env.HOME, ".node_libraries"));
+  if (process.env["NODE_PATH"]) {
+    modulePaths = process.env["NODE_PATH"].split(":").concat(modulePaths);
   }
-  defaultPaths.push(path.join(process.execPath, "..", "..", "lib", "node"));
 
   var extensions = {};
   var registerExtension = removed('require.registerExtension() removed. Use require.extensions instead');
@@ -171,7 +166,7 @@ var module = (function () {
   }
 
   function findModulePath (request, paths) {
-    var nextLoc = traverser(request, paths);
+    var nextLoc = traverser(request, request.charAt(0) === '/' ? [''] : paths);
 
     var fs = requireNative('fs');
 
@@ -183,29 +178,15 @@ var module = (function () {
     return false;
   }
 
-  function modulePathWalk (parent) {
-    if (parent._modulePaths) return parent._modulePaths;
-    var p = parent.filename.split("/");
-    var mp = [];
-    while (undefined !== p.pop()) {
-      mp.push(p.join("/")+"/node_modules");
-    }
-    return parent._modulePaths = mp;
-  }
 
   // sync - no i/o performed
   function resolveModuleLookupPaths (request, parent) {
 
     if (natives[request]) return [request, []];
 
-    if (request.charAt(0) === '/') {
-      return [request, ['']];
-    }
-
     var start = request.substring(0, 2);
     if (start !== "./" && start !== "..") {
-      var paths = modulePaths.concat(modulePathWalk(parent)).concat(defaultPaths);
-      return [request, paths];
+      return [request, modulePaths];
     }
 
     // Is the parent an index module?
@@ -425,45 +406,26 @@ var constants; // lazy loaded.
   };
 })();
 
-// Timers
-function addTimerListener (callback) {
-  var timer = this;
-  // Special case the no param case to avoid the extra object creation.
-  if (arguments.length > 2) {
-    var args = Array.prototype.slice.call(arguments, 2);
-    timer.callback = function () { callback.apply(timer, args); };
-  } else {
-    timer.callback = callback;
-  }
-}
 
-var Timer; // lazy load
-
-global.setTimeout = function (callback, after) {
-  if (!Timer) Timer = process.binding("timer").Timer;
-  var timer = new Timer();
-  addTimerListener.apply(timer, arguments);
-  timer.start(after, 0);
-  return timer;
+global.setTimeout = function () {
+  var t = module.requireNative('timers');
+  return t.setTimeout.apply(this, arguments);
 };
 
-global.setInterval = function (callback, repeat) {
-  if (!Timer) Timer = process.binding("timer").Timer;
-  var timer = new Timer();
-  addTimerListener.apply(timer, arguments);
-  timer.start(repeat, repeat ? repeat : 1);
-  return timer;
+global.setInterval = function () {
+  var t = module.requireNative('timers');
+  return t.setInterval.apply(this, arguments);
 };
 
-global.clearTimeout = function (timer) {
-  if (!Timer) Timer = process.binding("timer").Timer;
-  if (timer instanceof Timer) {
-    timer.callback = null;
-    timer.stop();
-  }
+global.clearTimeout = function () {
+  var t = module.requireNative('timers');
+  return t.clearTimeout.apply(this, arguments);
 };
 
-global.clearInterval = global.clearTimeout;
+global.clearInterval = function () {
+  var t = module.requireNative('timers');
+  return t.clearInterval.apply(this, arguments);
+};
 
 
 var stdout;
@@ -589,8 +551,8 @@ global.console.assert = function(expression){
 global.Buffer = module.requireNative('buffer').Buffer;
 
 process.exit = function (code) {
-  process.emit("exit");
-  process.reallyExit(code);
+  process.emit("exit", code || 0);
+  process.reallyExit(code || 0);
 };
 
 process.kill = function (pid, sig) {
@@ -620,19 +582,11 @@ if (process.argv[1]) {
 
 } else if (process._eval) {
     // -e, --eval
-    if (process._eval) console.log(eval(process._eval));
+    var indirectEval= eval; // so the eval happens in global scope.
+    if (process._eval) console.log(indirectEval(process._eval));
 } else {
     // REPL
   module.requireNative('repl').start();
 }
 
-// All our arguments are loaded. We've evaluated all of the scripts. We
-// might even have created TCP servers. Now we enter the main eventloop. If
-// there are no watchers on the loop (except for the ones that were
-// ev_unref'd) then this function exits. As long as there are active
-// watchers, it blocks.
-process.loop();
-
-process.emit("exit");
-
 });
