@@ -24,6 +24,9 @@ class Buffer : public ObjectWrap {
 
   typedef void (*free_callback)(char *data, void *hint);
 
+  // C++ API for constructing fast buffer
+  static v8::Handle<v8::Object> New(v8::Handle<v8::String> string);
+
   static void Initialize(v8::Handle<v8::Object> target);
   static Buffer* New(size_t length); // public constructor
   static Buffer* New(char *data, size_t len); // public constructor
@@ -34,17 +37,7 @@ class Buffer : public ObjectWrap {
   static char* Data(v8::Handle<v8::Object>);
   static size_t Length(v8::Handle<v8::Object>);
 
-  char* data() {
-    assert(0 && "v0.3 API change: Use node::Buffer::Data().");
-    return NULL;
-  }
-
-  size_t length() const {
-    assert(0 && "v0.3 API change: Use node::Buffer::Length().");
-    return 0;
-  }
-
- private:
+  private:
   static v8::Persistent<v8::FunctionTemplate> constructor_template;
 
   static v8::Handle<v8::Value> New(const v8::Arguments &args);
