@@ -39,6 +39,16 @@ static inline int SetNonBlocking(int fd) {
 }
 
 
+static inline int SetCloseOnExec(int fd) {
+  int flags = fcntl(fd, F_GETFD, 0);
+  int r = fcntl(fd, F_SETFD, flags | FD_CLOEXEC);
+  if (r != 0) {
+    perror("SetCloseOnExec()");
+  }
+  return r;
+}
+
+
 static inline int ResetFlags(int fd) {
   int flags = fcntl(fd, F_GETFL, 0);
   // blocking
@@ -223,6 +233,22 @@ int ChildProcess::Spawn(const char *file,
     return -1;
   }
 
+  // Set the close-on-exec FD flag
+  if (custom_fds[0] == -1) {
+    SetCloseOnExec(stdin_pipe[0]);
+    SetCloseOnExec(stdin_pipe[1]);
+  }
+
+  if (custom_fds[1] == -1) {
+    SetCloseOnExec(stdout_pipe[0]);
+    SetCloseOnExec(stdout_pipe[1]);
+  }
+
+  if (custom_fds[2] == -1) {
+    SetCloseOnExec(stderr_pipe[0]);
+    SetCloseOnExec(stderr_pipe[1]);
+  }
+
   // Save environ in the case that we get it clobbered
   // by the child process.
   char **save_our_env = environ;
