@@ -242,6 +242,27 @@ Handle<Value> SecureContext::Close(const Arguments& args) {
   return False();
 }
 
+char ssl_error_buf[512];
+
+static int serr(SSL *ssl, const char* func, int rv) {
+  if (rv >= 0) {
+    return rv;
+  }
+
+  int err = SSL_get_error(ssl, rv);
+  if (err != SSL_ERROR_WANT_WRITE &&
+      err != SSL_ERROR_WANT_READ) {
+    ERR_error_string_n(ERR_get_error(), &ssl_error_buf[0], sizeof(ssl_error_buf));
+    /* fprintf(stderr, "[%p] SSL: %s failed: (%d:%d) %s\n", ssl, func, err, rv, buf); */
+    return rv;
+  } else if (err == SSL_ERROR_WANT_WRITE) {
+    /* fprintf(stderr, "[%p] SSL: %s want write\n", ssl, func); */
+  } else if (err == SSL_ERROR_WANT_READ) {
+    /* fprintf(stderr, "[%p] SSL: %s want read\n", ssl, func); */
+  }
+
+  return 0;
+}
 
 void SecureStream::Initialize(Handle<Object> target) {
   HandleScope scope;
@@ -260,6 +281,7 @@ void SecureStream::Initialize(Handle<Object> target) {
   NODE_SET_PROTOTYPE_METHOD(t, "isInitFinished", SecureStream::IsInitFinished);
   NODE_SET_PROTOTYPE_METHOD(t, "verifyPeer", SecureStream::VerifyPeer);
   NODE_SET_PROTOTYPE_METHOD(t, "getCurrentCipher", SecureStream::GetCurrentCipher);
+  NODE_SET_PROTOTYPE_METHOD(t, "start", SecureStream::Start);
   NODE_SET_PROTOTYPE_METHOD(t, "shutdown", SecureStream::Shutdown);
   NODE_SET_PROTOTYPE_METHOD(t, "close", SecureStream::Close);
 
@@ -333,7 +355,7 @@ Handle<Value> SecureStream::EncIn(const Arguments& args) {
           String::New("Length is extends beyond buffer")));
   }
 
-  int bytes_written = BIO_write(ss->bio_read_, (char*)buffer_data + off, len);
+  int bytes_written = serr(ss->ssl_, "BIO_write", BIO_write(ss->bio_read_, (char*)buffer_data + off, len));
 
   if (bytes_written < 0) {
     if (errno == EAGAIN || errno == EINTR) return Null();
@@ -379,32 +401,19 @@ Handle<Value> SecureStream::ClearOut(const Arguments& args) {
 
   if (!SSL_is_init_finished(ss->ssl_)) {
     if (ss->is_server_) {
-      bytes_read = SSL_accept(ss->ssl_);
+      bytes_read =  serr(ss->ssl_, "SSL_accept:ClearOut", SSL_accept(ss->ssl_));
     } else {
-      bytes_read = SSL_connect(ss->ssl_);
+      bytes_read = serr(ss->ssl_, "SSL_connect:ClearOut", SSL_connect(ss->ssl_));
     }
     if (bytes_read < 0) {
-      int err;
-      if ((err = SSL_get_error(ss->ssl_, bytes_read)) == SSL_ERROR_WANT_READ) {
-        return scope.Close(Integer::New(0));
-      }
+      return ThrowException(Exception::Error(v8::String::New(ssl_error_buf)));
     }
     return scope.Close(Integer::New(0));
   }
 
-  bytes_read = SSL_read(ss->ssl_, (char*)buffer_data + off, len);
+  bytes_read = serr(ss->ssl_, "SSL_read:ClearOut", SSL_read(ss->ssl_, (char*)buffer_data + off, len));
   if (bytes_read < 0) {
-    int err = SSL_get_error(ss->ssl_, bytes_read);
-    if (err == SSL_ERROR_WANT_READ) {
-      return scope.Close(Integer::New(0));
-    }
-    // SSL read error
-    return scope.Close(Integer::New(-2));
-  }
-
-  if (bytes_read < 0) {
-    if (errno == EAGAIN || errno == EINTR) return Null();
-    return ThrowException(ErrnoException(errno, "read"));
+    return ThrowException(Exception::Error(v8::String::New(ssl_error_buf)));
   }
 
   return scope.Close(Integer::New(bytes_read));
@@ -460,7 +469,7 @@ Handle<Value> SecureStream::EncOut(const Arguments& args) {
           String::New("Length is extends beyond buffer")));
   }
 
-  int bytes_read = BIO_read(ss->bio_write_, (char*)buffer_data + off, len);
+  int bytes_read = serr(ss->ssl_, "BIO_read:EncOut", BIO_read(ss->bio_write_, (char*)buffer_data + off, len));
 
   return scope.Close(Integer::New(bytes_read));
 }
@@ -500,13 +509,23 @@ Handle<Value> SecureStream::ClearIn(const Arguments& args) {
   if (!SSL_is_init_finished(ss->ssl_)) {
     int s;
     if (ss->is_server_) {
-      s = SSL_accept(ss->ssl_);
+      s = serr(ss->ssl_, "SSL_accept:ClearIn", SSL_accept(ss->ssl_));
     } else {
-      s = SSL_connect(ss->ssl_);
+      s = serr(ss->ssl_, "SSL_connect:ClearIn", SSL_connect(ss->ssl_));
     }
+
+    if (s < 0) {
+      return ThrowException(Exception::Error(v8::String::New(ssl_error_buf)));
+    }
+
     return scope.Close(Integer::New(0));
   }
-  int bytes_written = SSL_write(ss->ssl_, (char*)buffer_data + off, len);
+
+  int bytes_written = serr(ss->ssl_, "SSL_write:ClearIn", SSL_write(ss->ssl_, (char*)buffer_data + off, len));
+
+  if (bytes_written < 0) {
+    return ThrowException(Exception::Error(v8::String::New(ssl_error_buf)));
+  }
 
   return scope.Close(Integer::New(bytes_written));
 }
@@ -570,6 +589,32 @@ Handle<Value> SecureStream::GetPeerCertificate(const Arguments& args) {
   return scope.Close(info);
 }
 
+Handle<Value> SecureStream::Start(const Arguments& args) {
+  HandleScope scope;
+  int rv;
+
+  SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
+
+  if (!SSL_is_init_finished(ss->ssl_)) {
+    if (ss->is_server_) {
+      rv = serr(ss->ssl_, "SSL_accept:Start", SSL_accept(ss->ssl_));
+    } else {
+      rv = serr(ss->ssl_, "SSL_connect:Start", SSL_connect(ss->ssl_));
+    }
+
+    if (rv < 0) {
+      return ThrowException(Exception::Error(v8::String::New(ssl_error_buf)));
+    }
+
+    if (rv == 1) {
+      return True();
+    } else {
+      return False();
+    }
+  }
+
+  return True();
+}
 
 Handle<Value> SecureStream::Shutdown(const Arguments& args) {
   HandleScope scope;
@@ -684,25 +729,23 @@ static void HexDecode(unsigned char *input,
 
 
 void base64(unsigned char *input, int length, char** buf64, int* buf64_len) {
-  BIO *bmem, *b64;
-  BUF_MEM *bptr;
-  int len;
-
-  b64 = BIO_new(BIO_f_base64());
-  bmem = BIO_new(BIO_s_mem());
+  BIO *b64 = BIO_new(BIO_f_base64());
+  BIO *bmem = BIO_new(BIO_s_mem());
   b64 = BIO_push(b64, bmem);
   BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
-  len = BIO_write(b64, input, length);
+  int len = BIO_write(b64, input, length);
   assert(len == length);
   int r = BIO_flush(b64);
   assert(r == 1);
+
+  BUF_MEM *bptr;
   BIO_get_mem_ptr(b64, &bptr);
 
   *buf64_len = bptr->length;
   *buf64 = new char[*buf64_len+1];
-  memcpy(*buf64, bptr->data, bptr->length);
+  memcpy(*buf64, bptr->data, *buf64_len);
   char* b = *buf64;
-  b[bptr->length] = 0;
+  b[*buf64_len] = 0;
 
   BIO_free_all(b64);
 }
@@ -1836,15 +1879,6 @@ class Hash : public ObjectWrap {
     return 1;
   }
 
-  int HashDigest(unsigned char** md_value, unsigned int *md_len) {
-    if (!initialised_) return 0;
-    *md_value = new unsigned char[EVP_MAX_MD_SIZE];
-    EVP_DigestFinal_ex(&mdctx, *md_value, md_len);
-    EVP_MD_CTX_cleanup(&mdctx);
-    initialised_ = false;
-    return 1;
-  }
-
 
  protected:
 
@@ -1903,22 +1937,27 @@ class Hash : public ObjectWrap {
   }
 
   static Handle<Value> HashDigest(const Arguments& args) {
-    Hash *hash = ObjectWrap::Unwrap<Hash>(args.This());
-
     HandleScope scope;
 
-    unsigned char* md_value;
+    Hash *hash = ObjectWrap::Unwrap<Hash>(args.This());
+
+    if (!hash->initialised_) {
+      return ThrowException(Exception::Error(String::New("Not initialized")));
+    }
+
+    unsigned char md_value[EVP_MAX_MD_SIZE];
     unsigned int md_len;
-    char* md_hexdigest;
-    int md_hex_len;
-    Local<Value> outString ;
 
-    int r = hash->HashDigest(&md_value, &md_len);
+    EVP_DigestFinal_ex(&hash->mdctx, md_value, &md_len);
+    EVP_MD_CTX_cleanup(&hash->mdctx);
+    hash->initialised_ = false;
 
-    if (md_len == 0 || r == 0) {
+    if (md_len == 0) {
       return scope.Close(String::New(""));
     }
 
+    Local<Value> outString;
+
     if (args.Length() == 0 || !args[0]->IsString()) {
       // Binary
       outString = Encode(md_value, md_len, BINARY);
@@ -1926,10 +1965,14 @@ class Hash : public ObjectWrap {
       String::Utf8Value encoding(args[0]->ToString());
       if (strcasecmp(*encoding, "hex") == 0) {
         // Hex encoding
+        char* md_hexdigest;
+        int md_hex_len;
         HexEncode(md_value, md_len, &md_hexdigest, &md_hex_len);
         outString = Encode(md_hexdigest, md_hex_len, BINARY);
         delete [] md_hexdigest;
       } else if (strcasecmp(*encoding, "base64") == 0) {
+        char* md_hexdigest;
+        int md_hex_len;
         base64(md_value, md_len, &md_hexdigest, &md_hex_len);
         outString = Encode(md_hexdigest, md_hex_len, BINARY);
         delete [] md_hexdigest;
@@ -1940,9 +1983,8 @@ class Hash : public ObjectWrap {
                         "can be binary, hex or base64\n");
       }
     }
-    delete [] md_value;
-    return scope.Close(outString);
 
+    return scope.Close(outString);
   }
 
   Hash () : ObjectWrap () {
