@@ -3,12 +3,13 @@ var path = require("path");
 exports.testDir = path.dirname(__filename);
 exports.fixturesDir = path.join(exports.testDir, "fixtures");
 exports.libDir = path.join(exports.testDir, "../lib");
+exports.tmpDir = path.join(exports.testDir, "tmp");
 exports.PORT = 12346;
 
 exports.assert = require('assert');
 
-var sys = require("sys");
-for (var i in sys) exports[i] = sys[i];
+var util = require("util");
+for (var i in util) exports[i] = util[i];
 //for (var i in exports) global[i] = exports[i];
 
 function protoCtrChain (o) {
