@@ -8,6 +8,10 @@ var WINDOW = 200; // why is does this need to be so big?
 var interval_count = 0;
 var setTimeout_called = false;
 
+// check that these don't blow up.
+clearTimeout(null);
+clearInterval(null);
+
 assert.equal(true, setTimeout instanceof Function);
 var starttime = new Date;
 setTimeout(function () {
