@@ -1,6 +1,6 @@
-require('../common');
-var exec = require('child_process').exec,
-    sys = require('sys');
+common = require('../common');
+assert = common.assert;
+var exec = require('child_process').exec;
 success_count = 0;
 error_count = 0;
 response = "";
