@@ -12,12 +12,12 @@ var responses_recvd = 0;
 var body0 = "";
 var body1 = "";
 
-var server = http.createServer(function (req, res) {
+var server = http.Server(function (req, res) {
   if (responses_sent == 0) {
     assert.equal("GET", req.method);
     assert.equal("/hello", url.parse(req.url).pathname);
 
-    common.p(req.headers);
+    console.dir(req.headers);
     assert.equal(true, "accept" in req.headers);
     assert.equal("*/*", req.headers["accept"]);
 
