@@ -1,7 +1,7 @@
 common = require("../common");
 assert = common.assert
 
-var Script = process.binding('evals').Script;
+var Script = require('vm').Script;
 var script = new Script('"passed";');
 
 common.debug('run in a new empty context');
@@ -19,3 +19,7 @@ script = new Script('foo = 3;');
 result = script.runInContext(context);
 assert.equal(3, context.foo);
 assert.equal('lala', context.thing);
+
+
+// Issue GH-227:
+Script.runInNewContext('', null, 'some.js');
