@@ -2,11 +2,15 @@ common = require("../common");
 assert = common.assert
 
 var url = require("url"),
-  sys = require("sys");
+  util = require("util");
 
 // URLs to parse, and expected data
 // { url : parsed }
 var parseTests = {
+  "//some_path" : {
+    "href": "//some_path",
+    "pathname": "//some_path"
+  },
   "http://www.narwhaljs.org/blog/categories?id=news" : {
     "href": "http://www.narwhaljs.org/blog/categories?id=news",
     "protocol": "http:",
